// Matrix class
// written by Stephan Senn, D-ITET
// This class is used for matrix operations with double types.
// 5.12.03

#include <stdlib.h>
#include <cmath>
#include <iostream>
#include "Matrix.h"
using namespace std;

Matrix::Matrix(int n, int m)
{
  matrix=new double*[n];
  N=n;
  M=m;

  for(int k=0;k<n;k++)
     matrix[k]=new double[m];
}

Matrix::Matrix()
{
    N=0;
    M=0;
    matrix=NULL;
}

Matrix::~Matrix()
{
    for(int l=0;l<M;l++)
	delete [] matrix[l];

    delete matrix;

}



Matrix* Matrix::add(Matrix* A, Matrix* B)
{
    Matrix* tmp=new Matrix(A->N,A->M);
    int N=A->N;
    int M=A->M;
    
    if(A->N!=B->N || B->M!=B->M)
	cout<<"Matrizes have wrong dimensions.\n";
    else
    {
    for(int i=0;i<N;i++)
	for(int j=0;j<M;j++)
	{
	    tmp->matrix[i][j]=A->matrix[i][j]+B->matrix[i][j];
	    cout<<tmp->matrix[i][j]<<endl;
	}
    }
    
    return tmp;

}

Matrix* Matrix::minus(Matrix* A, Matrix* B)
{
    Matrix* tmp=new Matrix(A->N,A->M);
    int N=A->N;
    int M=A->M;
    
    if(A->N!=B->N || B->M!=B->M)
	cout<<"Matrizes have wrong dimensions.\n";
    else
    {
    for(int i=0;i<N;i++)
	for(int j=0;j<M;j++)
	{
	    tmp->matrix[i][j]=A->matrix[i][j]-B->matrix[i][j];
	    cout<<tmp->matrix[i][j]<<endl;
	}
    }
    
    return tmp;

}

Matrix* Matrix::mult(Matrix* A, Matrix* B)
{
    Matrix* tmp=new Matrix(A->N,B->M);
    int s=A->N;
    int t=B->M;
    
    if(A->M!=B->N)
	cout<<"Matrizes have wrong dimensions.\n";
    else
    {
    for(int i=0;i<s;i++)
    {
	for(int j=0;j<t;j++)
	{
	    double sum=0.0;
	    for(int k=0;k<B->N;k++)
	    {
		sum=sum+A->matrix[i][k]*B->matrix[k][j];
	    }
	
	tmp->matrix[i][j]=sum;
	}
    }
    }
    
    return tmp;
}

Matrix* Matrix::transpose(Matrix* A)
{
    Matrix* tmp=new Matrix(A->M,A->N);

    for(int i=0;i<A->N;i++)
	for(int j=0;j<A->M;j++)
	    tmp->matrix[j][i]=A->matrix[i][j];

    return tmp;
}

void Matrix::copyDATA(Matrix* from, Matrix* to)
{
    if(from->N!=to->N || from->M!=to->M)
	cout<<"Matrizes have not the same dimensions.";
    else
    {
    for(int i=0;i<from->N;i++)
	for(int j=0;j<from->M;j++)
	    to->matrix[i][j]=from->matrix[i][j];
    }
}

void Matrix::printDATA(Matrix* A)
{
    for(int i=0;i<A->N;i++)
	for(int j=0;j<A->M;j++)
	    cout<<"["<<i<<","<<j<<"]="<<A->matrix[i][j]<<";"<<endl;
}

Matrix* Matrix::linearFit(Matrix* A, Matrix* y)
{
    Matrix* tA=A->transpose(A);
    Matrix* tAA=A->mult(tA,A);
    Matrix* tAy=y->mult(tA,y);

    Matrix* INVtAA=new Matrix(2,2);
    copyDATA(tAA,INVtAA);

    double det=tAA->matrix[0][0]*tAA->matrix[1][1]-tAA->matrix[1][0]*tAA->matrix[0][1];

    Matrix* tmp=new Matrix(2,1);

    if(det==0)
	cout<<"Line is at 90 therefore rise goes to infinity. No Solution.";
    else
    {
	INVtAA->matrix[0][0]=tAA->matrix[1][1]/det;
	INVtAA->matrix[1][1]=tAA->matrix[0][0]/det;
	INVtAA->matrix[1][0]=-tAA->matrix[1][0]/det;
	INVtAA->matrix[0][1]=-tAA->matrix[0][1]/det;
		    
	tmp=tmp->mult(INVtAA,tAy);
    }
    
    return tmp;
    
}
