// util class
// This is total renewed version!
// This class uses util.h as header file.
// written by Stephan Senn, D-ITET
// 22.1.04
// This class performs the recognition of the orientation of the playcard.

#include <iostream>
#include "imageio.h"
#include "util.h"
#include <cmath>
#include <stdlib.h>
#include "point2d.h"
#include "Matrix.h"
using namespace std;

// Constructor
util::util(char* name, bool save)
{
    readImage(name);
    si=save; // Do you wanna save your image on your harddisk?
}

util::util(bitmap picture, bool save)
{
    bmp=picture;
    si=save;
}

// Reading image from file...
void util::readImage(char* name)
{
    if(bmp.read(name)!=true)
	cout<<"Error reading file "<<name<<".\n";
}

// Saving image to file...
void util::saveImage(char* name)
{   
    if(bmp.write(name)!=true)
	cout<<"Error writing file "<<name<<".\n";
}

// For Color Filtering...
void util::colorFilter()
{
    // Scaling image...
    for(int j=0;j<bmp.width;j++)
	for(int i=0;i<bmp.heigth;i++)
	{
	    int r=bmp.map[i][j].r;
	    int g=bmp.map[i][j].g;
	    int b=bmp.map[i][j].b;
	    
	    // Now set Background to green color...
	    if((r>10 && r<100) & (g>10 && g<100) & (b>10 && b<100))
	    {
		bmp.map[i][j].r=0;
		bmp.map[i][j].g=255;
		bmp.map[i][j].b=0;

	    }

	    // Now set Background of the playcard to white color...
	    if((r>200) & (g>200) & (b>200))
	    {
		bmp.map[i][j].r=255;
		bmp.map[i][j].g=255;
		bmp.map[i][j].b=255;

	    }
	}
        
}

// Mainroutine for fetching the corners of the playcard...
point2d* util::fetchCorner(int step, bool trial)
{
    // First we filters the image with our Laplace Filter...
    laplaceFilter();

    // We set the threshold value normally to 300. This value 
    // can be found experimentally. When the picture is very bright due
    // to camera settings the threshhold value is 250.
    
    int threshhold=310;

    if(calcContrast()>200)
	threshhold=250;

    // Now we try to fetch the corners...
    cout<<"Try fetching corner position of playcard...";

    // Return value...
    point2d* re;

    // Storing Results...
    point2d* results_top = new point2d();
    point2d* head_top=results_top;
    int hit_top=0;

    point2d* results_down = new point2d();
    point2d* head_down=results_down;
    int hit_down=0;

    point2d* results_left = new point2d();
    point2d* head_left=results_left;
    int hit_left=0;

    point2d* results_right = new point2d();
    point2d* head_right=results_right;
    int hit_right=0;

    // Finding points on the border of the card...
    bmp.setPen(255,255,0);

    for(int j=5;j<bmp.width-5;j+=step)
	for(int i=5;i<bmp.heigth-5;i++)
	{
	    int df=diffcolor(i,j,i+1,j);
	    if(df>threshhold)
	    {
		results_top->next=new point2d((double)j,(double)i);
		results_top=results_top->next;
		hit_top++;
		bmp.dot(i,j,1);
		break;
	    }
	}

    for(int j=bmp.width-5;j>5;j-=step)
	for(int i=bmp.heigth-5;i>5;i--)
	{
	    int df=diffcolor(i,j,i-1,j);
	    if(df>threshhold)
	    {
		results_down->next=new point2d((double)j,(double)i);
		results_down=results_down->next;
		hit_down++;
		bmp.dot(i,j,1);
		break;
	    }
	}
    
    
    Matrix* M1=readInMatrix(head_top,hit_top);
    Matrix* M2=readInMatrix(head_down,hit_down);
    
    middlepoint=calcMiddlePoint(M1,M2);
    
    
    if(checkNormal(M1)==true)
    {
       int t1=(int)M1->matrix[5][1]+5;
       int t2=(int)M2->matrix[5][1]-5;

       for(int i=t1;i<t2;i+=step)
	for(int j=5;j<bmp.width-5;j++)
	{
	    int df=diffcolor(i,j,i,j+1);
	    if(df>threshhold)
	    {
		results_left->next=new point2d((double)j,(double)i);
		results_left=results_left->next;
		hit_left++;
		//bmp.dot(i,j,1);
		break;
	    }
	}

       for(int i=t1;i<t2;i+=step)
	for(int j=bmp.width-5;j>5;j--)
	{
	    int df=diffcolor(i,j,i,j-1);
	    if(df>threshhold)
	    {
		results_right->next=new point2d((double)j,(double)i);
		results_right=results_right->next;
		hit_right++;
		//bmp.dot(i,j,1);
		break;
	    }
	}

	Matrix* M3=readInMatrix(head_left,hit_left);
	Matrix* M4=readInMatrix(head_right,hit_right);

	re=evalNormalLines(M1,M2,M3,M4);
    }
    else
    {
    
	int position1=dividePosition(M1,middlepoint);
	int position2=dividePosition(M2,middlepoint);
	Matrix* M11=divideIntoFirstPart(M1,position1);
	Matrix* M12=divideIntoSecondPart(M1,position1);
	Matrix* M21=divideIntoFirstPart(M2,position2);
	Matrix* M22=divideIntoSecondPart(M2,position2);
    
    
    int c11=M11->N;
    int c12=M12->N;
    int c21=M21->N;
    int c22=M22->N;

    if(c11<8 || c12<8)
    {
	if(c11<c12)
	    M11=NULL;
	else
	    M12=NULL;

	if(c21<c22)
	    M21=NULL;
	else
	    M22=NULL;

      for(int i=5;i<bmp.heigth;i+=step)
	for(int j=5;j<bmp.width-5;j++)
	{
	    int df=diffcolor(i,j,i,j+1);
	    if(df>threshhold)
	    {
		results_left->next=new point2d((double)j,(double)i);
		results_left=results_left->next;
		hit_left++;
		//bmp.dot(i,j,1);
		break;
	    }
	}

       Matrix* M3=readInMatrix(head_left,hit_left);
       int position3=dividePosition(M3,middlepoint);
       Matrix* M31=divideIntoFirstPart(M3,position3);
       Matrix* M32=divideIntoSecondPart(M3,position3);
       Matrix* tmp;

       int c31=M31->N;
       int c32=M32->N;

       if(c31>c32)
	   tmp=M31;
       else
	   tmp=M32;

       if(M11==NULL)
	   M11=tmp;
       else
	   M12=tmp;

       for(int i=bmp.heigth-5;i>5;i-=step)
	for(int j=bmp.width-5;j>5;j--)
	{
	    int df=diffcolor(i,j,i,j-1);
	    if(df>threshhold)
	    {
		results_right->next=new point2d((double)j,(double)i);
		results_right=results_right->next;
		hit_right++;
		//bmp.dot(i,j,1);
		break;
	    }
	}
       
       Matrix* M4=readInMatrix(head_right,hit_right);
       int position4=dividePosition(M4,middlepoint);
       Matrix* M41=divideIntoFirstPart(M4,position4);
       Matrix* M42=divideIntoSecondPart(M4,position4);

       int c41=M41->N;
       int c42=M42->N;

       if(c41>c42)
	   tmp=M41;
       else
	   tmp=M42;

       if(M21==NULL)
	   M21=tmp;
       else
	   M22=tmp;
    }   

    calcLine(M11,0);
    calcLine(M12,1);
    calcLine(M21,2);
    calcLine(M22,3);

    // Now try to calculate the corners...
    point2d* k3=calcISection(lines[0],lines[1]);
    point2d* k4=calcISection(lines[1],lines[2]);
    point2d* k1=calcISection(lines[2],lines[3]);
    point2d* k2=calcISection(lines[3],lines[0]);
    
    // Making corners and borders visible in the image...
    bmp.setPen(255,0,0);
    bmp.dot(k1->Y(),k1->X(),2);
    bmp.dot(k2->Y(),k2->X(),2);
    bmp.dot(k3->Y(),k3->X(),2);
    bmp.dot(k4->Y(),k4->X(),2);

    bmp.setPen(255,0,0);
    bmp.line(k1->Y(),k1->X(),k2->Y(),k2->X());
    bmp.line(k2->Y(),k2->X(),k3->Y(),k3->X());
    bmp.line(k3->Y(),k3->X(),k4->Y(),k4->X());
    bmp.line(k1->Y(),k1->X(),k4->Y(),k4->X());
    

    // Now calculate the right order... (for recognizing the card itself)
    if(point.distance(k1,k2)>point.distance(k2,k3))
    {
	point2d* t1=new point2d(k1->x,k1->y);
	delete k1;
	point2d* t2=new point2d(k2->x,k2->y);
	delete k2;
	point2d* t3=new point2d(k3->x,k3->y);
	delete k3;
	point2d* t4=new point2d(k4->x,k4->y);
	delete k4;

	k1=t2;
	k2=t3;
	k3=t4;
	k4=t1;	
    }

    // Packing corners for returning them as pointers...
    re=k1;
    re->next=k2;
    re=re->next;
    re->next=k3;
    re=re->next;
    re->next=k4;
    re=k1;

    // Finish
    cout<<"\tDone."<<endl;

     // Card verification...
    if(verifyCard(k1,k2,k3,k4)==false && trial==true)
    {
        cout<<"Try fetching corner with other parameters..."<<endl;
	re=fetchCorner(8,false);
    }
   
    }

    // If you wanna save your results...
    if(si==true)
	saveImage("util_output_image.pnm");
    else
	bmp.map=NULL;

    
    return re;

}

// For verifying the found corner positions...
bool util::verifyCard(point2d* k1, point2d* k2, point2d* k3, point2d* k4)
{
    cout<<"Verifying corner positions of the playcard...";
    double a=point.distance(k1,k2);
    double b=point.distance(k2,k3);
    double d=point.distance(k1,k4);
    double c=point.distance(k4,k3);
    double d1=point.distance(k1,k3);
    double d2=point.distance(k2,k4);
    double diff=abs(d1-d2);

    bool re=false;
    double q1;
    double q2;

    if(a>b)
	q1=b/a;
    else
	q2=a/b;

    if(c>d)
	q2=d/c;
    else
	q2=c/d;

    if(q1>0.6 && q1<0.7)
	re=true;
    else
	re=false;

    if(q2>0.6 && q2<0.7)
	re=true;
    else
	re=false;


    if(diff<3)
	re=true;
    else
	re=false;

    if(re==true)
	cout<<"\tDone."<<endl;
    else
	cout<<"\tFailed."<<endl;

    return re;
}

// For recognizing corners with boarders parallel to the boarders of the image..
point2d* util::evalNormalLines(Matrix* top, Matrix* down, Matrix* left, Matrix* right)
{
    double y1=0;
    double count_y1=0;
    double sum_y1=0;
    double y2=0;
    double count_y2=0;
    double sum_y2=0;
    double x1=0;
    double count_x1=0;
    double sum_x1=0;
    double x2=0;
    double count_x2=0;
    double sum_x2=0;

    for(int i=4;i<top->N-4;i++)
    {
	    sum_y1=sum_y1+top->matrix[i][1];
	    count_y1++;
    }

    for(int i=4;i<down->N-4;i++)
    {
	    sum_y2=sum_y2+down->matrix[i][1];
	    count_y2++;
    }

    for(int i=4;i<left->N-4;i++)
    {
	    sum_x1=sum_x1+left->matrix[i][0];
	    count_x1++;
    }

    for(int i=4;i<right->N-4;i++)
    {
	    sum_x2=sum_x2+right->matrix[i][0];
	    count_x2++;
    }

    // Calculating the coordinates...
    y1=sum_y1/count_y1;
    y2=sum_y2/count_y2;
    x1=sum_x1/count_x1;
    x2=sum_x2/count_x2;

    // Building the coordinates as point2d types...
    point2d* p1=new point2d(x1,y1);
    point2d* p2=new point2d(x2,y1);
    point2d* p3=new point2d(x2,y2);
    point2d* p4=new point2d(x1,y2);

    bmp.setPen(255,0,0);

    // Making corners visible in the image...
    bmp.dot(p1->Y(),p1->X(),2);
    bmp.dot(p2->Y(),p2->X(),2);
    bmp.dot(p3->Y(),p3->X(),2);
    bmp.dot(p4->Y(),p4->X(),2);

    // Making borders visible in the image...
    bmp.setPen(255,0,0);
    bmp.line(p1->Y(),p1->X(),p2->Y(),p2->X());
    bmp.line(p2->Y(),p2->X(),p3->Y(),p3->X());
    bmp.line(p3->Y(),p3->X(),p4->Y(),p4->X());
    bmp.line(p1->Y(),p1->X(),p4->Y(),p4->X());

     // Now calculate the right order... (for recognizing the card itself)
    if(point.distance(p1,p2)>point.distance(p2,p3))
    {
	point2d* t1=new point2d(p1->x,p1->y);
	delete p1;
	point2d* t2=new point2d(p2->x,p2->y);
	delete p2;
	point2d* t3=new point2d(p3->x,p3->y);
	delete p3;
	point2d* t4=new point2d(p4->x,p4->y);
	delete p4;

	p1=t4;
	p2=t1;
	p3=t2;
	p4=t3;	
    }

    // Giving result back in a simple chained list...
    point2d* r=p1;
    p1->next=p2;
    p2->next=p3;
    p3->next=p4;

    return r;
}

// Checks whether the orientation of the playcard is normal to the boarder of
// image or not.
bool util::checkNormal(Matrix* m)
{
    double sumX=0.0;
    double averX=0.0;
    double averY=0.0;
    
    double sumY=0.0;
    bool check=false;

    for(int i=4;i<m->N-5;i++)
    {
	    sumX=sumX+abs(m->matrix[i][0]-m->matrix[i+1][0]);       
	    sumY=sumY+abs(m->matrix[i][1]-m->matrix[i+1][1]);
    }
    
    averX=abs(sumX/(m->N-10));
    averY=abs(sumY/(m->N-10));
    
    if(averX<0.5 || averY<0.5)
	check=true;

    return check;
}

// Calculates the intersection point of two lines...
point2d* util::calcISection(borderline b1, borderline b2)
{
        double x=(b1.offset-b2.offset)/(b2.rise-b1.rise);
        double y=(b1.rise*b2.offset-b1.offset*b2.rise)/(b1.rise-b2.rise);
        return new point2d(x,y);
        
}

// Calculates the rise and the offset of a line with given points...
void util::calcLine(Matrix* m, int line)
{
    Matrix* A=new Matrix(m->N,2);
    Matrix* y=new Matrix(m->N,1);

    for(int i=0;i<m->N;i++)
    {
	A->matrix[i][0]=m->matrix[i][0];
	A->matrix[i][1]=1;
	y->matrix[i][0]=m->matrix[i][1];
    }

    Matrix* r=m->linearFit(A,y);
    lines[line].rise=r->matrix[0][0];
    lines[line].offset=r->matrix[1][0];
}

// Read in the points that you have found into a matrix...
Matrix* util::readInMatrix(point2d* head, int length)
{
    Matrix* tmp=new Matrix(length-3,2);
    int i=0;
    point2d* p=head->next;
    p=p->next;
    p=p->next;

    for(;i<length-3;p=p->next,i++)
    {
	tmp->matrix[i][0]=p->x;
	tmp->matrix[i][1]=p->y;
    }

    return tmp;
}

// Divide the matrix...
Matrix* util::divideIntoFirstPart(Matrix* m, int position)
{
    Matrix* tmp;

    position=position-4;

    if(position<0)
	tmp=new Matrix(0,0);
    else
    {
    int l=position+1;

    tmp=new Matrix(l,2);

    for(int i=0;i<l;i++)
    {
	tmp->matrix[i][0]=m->matrix[i][0];
	tmp->matrix[i][1]=m->matrix[i][1];
    }
    }
    return tmp;
}


// Divide the matrix...
Matrix* util::divideIntoSecondPart(Matrix* m, int position)
{
    Matrix* tmp;
    position=position+4;

    int l=m->N-(position+1);

    if(l<0)
	tmp=new Matrix(0,0);
    else
    {
    tmp=new Matrix(l,2);

    for(int i=0;i<l;i++)
    {
	tmp->matrix[i][0]=m->matrix[i+position][0];
	tmp->matrix[i][1]=m->matrix[i+position][1];
    }
    }

    return tmp;
}



int util::dividePosition(Matrix* m, point2d p)
{
    double x=p.x;
    double y=p.y;
    int position=0;

    double dist=(x-m->matrix[0][0])*(x-m->matrix[0][0])+(y-m->matrix[0][1])*(y-m->matrix[0][1]);
    int pos1=0;

    for(int i=0;i<8;i++)
    {
	double d=(x-m->matrix[i][0])*(x-m->matrix[i][0])+(y-m->matrix[i][1])*(y-m->matrix[i][1]);
	if(d>dist)
	{
	    dist=d;
	    pos1=i;
	}
    }

    dist=(x-m->matrix[7][0])*(x-m->matrix[7][0])+(y-m->matrix[7][1])*(y-m->matrix[7][1]);
    int pos2=8;

    for(int i=8;i<m->N-8;i++)
    {
	double d=(x-m->matrix[i][0])*(x-m->matrix[i][0])+(y-m->matrix[i][1])*(y-m->matrix[i][1]);
	if(d>dist)
	{
	    dist=d;
	    pos2=i;
	}
    }

    int pos3=m->N-8;
    dist=(x-m->matrix[pos3][0])*(x-m->matrix[pos3][0])+(y-m->matrix[pos3][1])*(y-m->matrix[pos3][1]);

    for(int i=m->N-8;i<m->N;i++)
    {
	double d=(x-m->matrix[i][0])*(x-m->matrix[i][0])+(y-m->matrix[i][1])*(y-m->matrix[i][1]);
	if(d>dist)
	{
	    dist=d;
	    pos3=i;
	}
    }

    if((pos1<2) && (pos3>m->N-3))
	position=pos2;
    else
    {
    if(pos1>1)
	position=pos1;

    if(pos3<m->N-2)
	position=pos3;
    }

    return position;
}

point2d util::calcMiddlePoint(Matrix* a, Matrix* b)
{
    int n=a->N+b->N;
    double sumx=0.0;
    double sumy=0.0;

    for(int i=0;i<a->N;i++)
	sumx=sumx+a->matrix[i][0];

    for(int i=0;i<b->N;i++)
	sumx=sumx+b->matrix[i][0];

    for(int i=0;i<a->N;i++)
	sumy=sumy+a->matrix[i][1];

    for(int i=0;i<b->N;i++)
	sumy=sumy+b->matrix[i][1];

    double x=sumx/n;
    double y=sumy/n;
    point2d p(x,y);

    return p;

}

// Laplace Filtering...
void util::laplaceFilter()
{
  // We filters the image with our laplace filter...
        cout<<"Try filtering image with a laplace filter...";

  const int w=bmp.width;
  const int h=bmp.heigth;
  const int A=w+2;
  const int B=h+2;
  
  // This is the mask for filtering...
  int mask[3][3]=
      { {0,1,0},
	{1,-2,1},
	{0,1,0}
      };

  // Arrays which stores image data for processing...
  int** olddata_red=new int*[B];
  int** olddata_green=new int*[B];
  int** olddata_blue=new int*[B];
  int** newdata_red=new int*[B];
  int** newdata_green=new int*[B];
  int** newdata_blue=new int*[B];

  for(int k=0;k<B;k++)
  {
     olddata_red[k]=new int[A];
     olddata_green[k]=new int[A];
     olddata_blue[k]=new int[A];
     newdata_red[k]=new int[A];
     newdata_green[k]=new int[A];
     newdata_blue[k]=new int[A]; 
  }

  // Initialize the olddata arrays...
  for(int z=0;z<w+2;z++)
    for(int t=0;t<h+2;t++)
      { 
        olddata_red[t][z]=0;
	olddata_green[t][z]=0;
	olddata_blue[t][z]=0;
        newdata_red[t][z]=0;
	newdata_green[t][z]=0;
	newdata_blue[t][z]=0;
      }   

  // Load the image to the olddata arrays...
  for(int x=0;x<w;x++)
    for(int y=0;y<h;y++)
      {
	olddata_red[y+1][x+1]=bmp.map[y][x].r;
	olddata_green[y+1][x+1]=bmp.map[y][x].g;
	olddata_blue[y+1][x+1]=bmp.map[y][x].b;
      }

  // For correct margin filtering...
  for(int a=0;a<h;a++)
    { 
      olddata_red[a+1][0]=bmp.map[a][0].r;
      olddata_red[a+1][w+1]=bmp.map[a][w-1].r;
      olddata_green[a+1][0]=bmp.map[a][0].g;
      olddata_green[a+1][w+1]=bmp.map[a][w-1].g;
      olddata_blue[a+1][0]=bmp.map[a][0].b;
      olddata_blue[a+1][w+1]=bmp.map[a][w-1].b;
    }

  for(int b=0;b<w;b++)
    { 
     olddata_red[0][b+1]=bmp.map[0][b].r;
     olddata_red[h+1][b+1]=bmp.map[h-1][b].r;
     olddata_green[0][b+1]=bmp.map[0][b].g;
     olddata_green[h+1][b+1]=bmp.map[h-1][b].g;
     olddata_blue[0][b+1]=bmp.map[0][b].b;
     olddata_blue[h+1][b+1]=bmp.map[h-1][b].b;
    }

  // Now filter the image with the mask...
  for(int x=1;x<w+1;x++)
    { 
	for(int y=1;y<h+1;y++)
	{ 
	  int sum2_red=0;
	  int sum2_green=0;
	  int sum2_blue=0;
        
        for(int j=0;j<3;j++)
        { 
	    int sum1_red=0;
	    int sum1_green=0;
	    int sum1_blue=0;

	  for(int i=0;i<3;i++)
	  {  
             sum1_red=sum1_red+olddata_red[y-i+1][x-j+1]*mask[j][i];
	     sum1_green=sum1_green+olddata_green[y-i+1][x-j+1]*mask[j][i];
	     sum1_blue=sum1_blue+olddata_blue[y-i+1][x-j+1]*mask[j][i];
	  }

         sum2_red=sum2_red+sum1_red;
	 sum2_green=sum2_green+sum1_green;
	 sum2_blue=sum2_blue+sum1_blue;

       }
	newdata_red[y][x]=sum2_red;
	newdata_green[y][x]=sum2_green;
	newdata_blue[y][x]=sum2_blue;
	
      }
    }

  // Now correct the filtered image by shifting the color channels...
  int min_value_red=newdata_red[0][0];
  int min_value_green=newdata_green[0][0];
  int min_value_blue=newdata_blue[0][0];
 
  for(int e=1;e<w+1;e++)
    for(int f=1;f<h+1;f++)
    {
       if(min_value_red>=newdata_red[f][e])
  	 min_value_red=newdata_red[f][e];

       if(min_value_green>=newdata_green[f][e])
  	   min_value_green=newdata_green[f][e];

       if(min_value_blue>=newdata_blue[f][e])
  	   min_value_blue=newdata_blue[f][e];
      }

  // Now write all data back to bmp.map...
  for(int x=0;x<w;x++)
    for(int y=0;y<h;y++)
      {
	bmp.map[y][x].r=newdata_red[y+1][x+1]-min_value_red;
	bmp.map[y][x].g=newdata_green[y+1][x+1]-min_value_green;
	bmp.map[y][x].b=newdata_blue[y+1][x+1]-min_value_red;
      }

  // Now clean up memory...
    for(int l=0;l<B;l++)
    {
	delete [] olddata_red[l];
	delete [] olddata_green[l];
	delete [] olddata_blue[l];
	delete [] newdata_red[l];
	delete [] newdata_green[l];
	delete [] newdata_blue[l];
    }

    	delete olddata_red;
	delete olddata_green;
	delete olddata_blue;
	delete newdata_red;
	delete newdata_green;
	delete newdata_blue;

   // Finish filtering...
        cout<<"\tDone."<<endl;
        
} 

double util::calcContrast()
{
    cout<<"Analyzing contrast level of image...";
    double sum=0.0;
    int count=0;
    for(int i=5;i<bmp.heigth;i++)
    {
	sum=sum+bmp.map[i][5].r+bmp.map[i][5].b+bmp.map[i][5].g;
	sum=sum+bmp.map[i][bmp.width-5].r+bmp.map[i][bmp.width-5].b+bmp.map[i][bmp.width-5].g;
	count+=2;
    }
    sum=sum/(double)count;

    cout<<"\t\tDone."<<endl;

    return sum;
	
}
// To calculate the total change between two color points...
int util::diffcolor(int i1,int j1,int i2,int j2)
{
    int tmp=0;
    tmp=abs(bmp.map[i1][j1].r-bmp.map[i2][j2].r);
    tmp=tmp+abs(bmp.map[i1][j1].g-bmp.map[i2][j2].g);
    tmp=tmp+abs(bmp.map[i1][j1].b-bmp.map[i2][j2].b);

    
    return tmp;
    
}

// Black Filtering...
void util::blackFilter()
{

    // Scaling image...
    for(int j=0;j<bmp.width;j++)
	for(int i=0;i<bmp.heigth;i++)
	{
	    int r=bmp.map[i][j].r;
	    int g=bmp.map[i][j].g;
	    int b=bmp.map[i][j].b;
	    
	    // Now set all pixels that aren't black to white...
	    if((r>80) || (g>80) || (b>80))
	    {
		bmp.map[i][j].r=255;
		bmp.map[i][j].g=255;
		bmp.map[i][j].b=255;

	    }

	}
        
}


// Deconstructor
util::~util()
{
    if(bmp.map!=NULL && si==true)
	saveImage("util_output_image.pnm");
    else
	bmp.map=NULL;
}

